// counttable.cpp
// v0.84 (7.3.2002)
// (c) 2002 Andreas Krueger
//
// Please, if you change anything in this code,
// mark the end of those lines with '// *+* your name *+* ' 
// (so that I can find it) and send the code 
// to cye-code@AndreasKrueger.de
//
// last change: 
//          by:


#include "counttable.h"
#include "helpers.h"

#include <qstring.h>
#include <qfontmetrics.h> 
#include <qmessagebox.h> 
#include <qtextstream.h> 
#include <qhbox.h> 

CountTable::CountTable( uint numRows, uint numCols, QWidget *parent, const char *name )
        : QVBox(parent, name )
{
	theZeroCells = new AutoErasedCells; // MUST BE first!

	QLabel * labelTitle = new QLabel (this);
	labelTitle ->setFont(QFont("times",18));
	labelTitle ->setAlignment(AlignVCenter|AlignHCenter);
	labelTitle ->setText("These coins are in your purse");
	
	ct=new QTable(numRows, numCols, this);
	// TODO: Inherit myQTable from QTable with other edit-direction
	//       and change cellCoord nextCellForEditing(uint r, uint c)

	QHBox * sumbox=new QHBox(this);
	labelPurseSum = new QLabel(sumbox);
	labelPurseSum->setFont(QFont("times",18));
	labelPurseSum->setAlignment(AlignRight|AlignVCenter);

	QPushButton *buttonReset = new QPushButton ("RESET\nthis table to 0",sumbox,"reset");
	connect( buttonReset, SIGNAL(clicked()), this, SLOT(reset()) );
	connect( buttonReset, SIGNAL(clicked()), this, SLOT(buttonResetPressed()) );


	// the countries on the left hand side
	QHeader *headerLeft =ct->verticalHeader ();
	
    QFont font(ct->font());
	font.setWeight( QFont::Bold ); // Boldface when selected
	QFontMetrics fm(font);

	int cellWidth=0, thisWidth;

	int i;
	for (i=1; i<=headerLeft->count();i++)
	{
		headerLeft->setLabel ( i-1, countrynamesWords[i]) ;
		thisWidth=fm.width((QString)countrynamesWords[i]);
		cellWidth = cellWidth<thisWidth ? thisWidth:cellWidth;
	}
	cellWidth +=fm.width("ix"); // to account for bigger width
	ct->setLeftMargin(cellWidth);

	// the coins on the top side
	QHeader *headerTop =ct->horizontalHeader ();
	cellWidth =fm.width(" 1234 ");
	for (i=1; i<=headerTop->count();i++)
	{
		headerTop->setLabel ( i-1, coinnames[i-1]) ;
		ct->setColumnWidth ( i-1, cellWidth );
	}
	ct->setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding , QSizePolicy::Fixed));

	connect( ct, SIGNAL(valueChanged(int,int)), this, SLOT(cellEdited(int,int)) );
	
	connect( ct, SIGNAL(clicked ( int, int , int , const QPoint & )), 
		this, SLOT(cellClicked( int, int , int , const QPoint & )) );

	connect( this, SIGNAL(sumChanged()), this, SLOT(sumUpAndDisplay()) );
	//reset();
	sumUpAndDisplay();

	// the cell sizes should NOT be changeable!
	RowHeight=	ct->verticalHeader()->sectionSize(3);
	ColWidth =	ct->horizontalHeader()->sectionSize(3);
	connect(ct->verticalHeader(), SIGNAL(sizeChange ( int, int, int) ),
		this, SLOT(resetSizeChangeRows( int , int , int ) ) );
	connect(ct->horizontalHeader(), SIGNAL(sizeChange ( int, int, int) ),
		this, SLOT(resetSizeChangeCols( int , int , int ) ) );
}

void CountTable::resetSizeChangeRows( int section, int oldSize, int newSize ) {
	ct->verticalHeader()->resizeSection(section, RowHeight);
	doNothing(oldSize); doNothing(newSize); // to avoid gcc warning
}
void CountTable::resetSizeChangeCols( int section, int oldSize, int newSize ) {
	ct->horizontalHeader()->resizeSection(section, ColWidth);
	doNothing(oldSize); doNothing(newSize); // to avoid gcc warning
}

void CountTable::buttonResetPressed(){
	emit Event("Reset");
}

vector<vector<Q_UINT32> > CountTable::toVectorArray(){
	vector<vector<Q_UINT32> > array;
	array.resize(NumCoins);
	uint coin, country; Q_UINT32 CoinPileHeight; bool ok;
	for (coin=0; coin<NumCoins;coin++){
		array[coin].resize(NumCountries);
		for (country=0; country<NumCountries;country++){
			ok=value(country, coin, CoinPileHeight);
			array[coin][country]=CoinPileHeight;
		}
	}
	return array;
}

void CountTable::setFromVectorArray(vector<vector<Q_UINT32> > array){
	uint coin, country; Q_UINT32 CoinPileHeight; 
	for (coin=0; coin<NumCoins;coin++){
		for (country=0; country<NumCountries;country++){
			CoinPileHeight=array[coin][country];
			setValue(country, coin, CoinPileHeight);
		}
	}
	emit sumChanged();
}



void CountTable::sumUpAndDisplay(){
	Q_UINT32 cellValue;
	sum=0;
	for (int row=0;row< ct->numRows();row++){
		for (int column=0;column< ct->numCols();column++){
			value((uint)row,(uint)column,cellValue);
			sum+=cellValue * coinvalues[column];
		}
	}
	QString str="", zero="";
	if ( ((long)(sum*100))%100 == 0 ) 
		zero=".00";
	else 
		if ( ((long)(sum*100))%10 == 0 ) zero="0";
	QTextOStream (&str) <<" "<<sum<<zero<<" Euro ";
	labelPurseSum->setText(str);

	emit theSum (sum);
}

QString CountTable::cellCoordName(uint row, uint col){
	QString str;
	QTextOStream(&str) <<"("<<col<<","<<row<<")";
	return str;
}

void CountTable::cellClicked ( int row, int col, int button, const QPoint & mousePos ) {
    int b;b=button; QPoint mP;mP=mousePos;

    emit pleaseChangeToCoinOfCell((uint)row, (uint)col);
	QString eventname="CellClicked";
	eventname+=cellCoordName((uint)row, (uint)col);
	emit Event(eventname);
	// qDebug("CountTable::cellClicked ");
}

void CountTable::cellEdited(int row,int column){
	bool ok;
	QString cellContent=ct->text(row, column);
	if (cellContent!="") {
		int result=cellContent.toInt( &ok, 10);
		if (!ok || result<0) {
			QMessageBox::information( this, "Euro Coins",
                            "Please type in positive NUMBERS only!\n"
                            "The field will be set to 0 now." );
			result=0;
			setFocusToCell(row, column);
		}
		setValue((uint)row, (uint)column, result);
	}

	QString eventname="CellEdited";
	eventname+=cellCoordName((uint)row, (uint)column);
	emit Event(eventname);

	cellCoord next=nextCellForEditing(row, column);

	emit pleaseChangeToCoinOfCell(next.row, next.column);
	emit sumChanged();
}




void CountTable::test1(){
	QString temp; 
	for (uint row=0;row<NumCountries;row++){
		for (uint column=0;column<NumCoins;column++){
			temp="";
			QTextOStream(&temp) <<row<<", "<<column;
			ct->setText (row, column, temp);
		}
	}
}

void CountTable::reset(){
	for (int row=0;row<ct->numRows();row++){
		for (int column=0;column<ct->numCols();column++){
			setValue((uint)row,(uint)column,0);
			//ct->setText (row, column, "");
		}
	}
	emit sumChanged();
}

bool CountTable::coordinatesOK(uint row, uint column){
  //return (!(row<0 || row > ct->numRows() || column <0 || column > ct->numCols()) );
  return (!( (int) row > ct->numRows() || (int) column > ct->numCols()) );
}


bool CountTable::value(uint row, uint column, Q_UINT32 &result){
	if (!coordinatesOK(row, column)) return false;
	QString cellContent;
	cellContent=ct->text(row, column);
	if (cellContent=="") {
		result=0;
		return true;
	}
	else {
		bool ok;
		result=cellContent.toInt( &ok, 10);
		return ok;
	}
}

bool CountTable::setValue(uint row, uint column, Q_UINT32 newValue) {
	if (!coordinatesOK(row, column)) return false;

	if (theZeroCells->isActive(cellCoord(row,column)))
		theZeroCells->remove(cellCoord(row,column));

	if (newValue<=0) {
		zeroCell(row, column);
		emit sumChanged();
		return true;
	}

	QString str;
	QTextOStream(&str) << newValue;
	ct->setText(row, column, str);
	
	emit sumChanged();
	return true;
}

void CountTable::setFocusToCell(uint row, uint column){
	ct->clearSelection( true ) ;
	ct->setCurrentCell ( row, column ) ;

	QString eventname="CellFocussed";
	eventname+=cellCoordName((uint)row, (uint)column);
	emit Event(eventname);
}


void CountTable::increaseCell(uint row, uint column) {
	if (coordinatesOK(row, column)){
		Q_UINT32 cellValue;
		bool ok = value(row,column, cellValue);
		if (!ok) 
			cellValue=0;
		setValue(row,column, cellValue+1);
		//setFocusToCell(row, column);
	}
}

void CountTable::decreaseCell(uint row, uint column) {
	if (coordinatesOK(row, column)){
		Q_UINT32 cellValue;
		bool ok = value(row,column, cellValue);
		if (!ok || cellValue==0) cellValue=1;
		setValue(row,column, cellValue-1);
		//setFocusToCell(row, column);
	}

}


void CountTable::zeroCell(uint row, uint column) {
	if (coordinatesOK(row, column)){
		if (theZeroCells->isActive(cellCoord(row, column)))
			theZeroCells->remove(cellCoord(row, column));
		ct->setText(row, column, "0");
		theZeroCells->insert(cellCoord(row, column));
		connect( theZeroCells, SIGNAL(eraseCell(cellCoord)), this, SLOT(setEmpty(cellCoord) ));
	}
}

void CountTable::setEmpty(cellCoord theCell){
	ct->setText(theCell.row, theCell.column, "");
}


void CountTable::test(){
	zeroCell(1,1);
}



void CountTable::test2(){
	Q_UINT32 result=0; bool ok=false;
	ok=value(1,1,result);
	QString str;
	QTextOStream(&str) << "bool: "<<ok<<"   value:"<<result;
	QMessageBox::information( this, "Application name",str  );
	ok=setValue(1,1,++result);
	str="";
	QTextOStream(&str) << "bool: "<<ok<<"   value:"<<result;
	QMessageBox::information( this, "Application name",str  );

}


//QSizePolicy CountTable::sizePolicy() const
//{
//    return QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );
//	//return QSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
//}












